///////////////////////////////////////////////////////////////
//  client.cpp - demonstrate protocol class                  //
//  ver 1.1                                                  //
//                                                           //
//  This file represents client code, which knows only       //
//  about the protocol class, e.g., the protocol language    //
//  and the names, but not the details, of the derived       //
//  classes.                                                 //
///////////////////////////////////////////////////////////////
//  build process                                            //
///////////////////////////////////////////////////////////////
//  Required files:                                          //
//    client.cpp, protocol.h, dynLL.lib, dynLL.dll           //
//                                                           //
//  To build in integrated environment:                      //
//    - create Win32 Console Application project             //
//    - add client.cpp and protocol.h                        //
//    - select Project\Settings\Link                         //
//    - add dynLL.lib to Object/library modules: edit box    //
//    - copy dynLL.dll and dynLL.lib  to client's project    //
//      directory from its debug directory                   //
//    - build and run                                        //
///////////////////////////////////////////////////////////////

#include <iostream>
#include "protocol.h"     // note that the client only needs
                          // the base class header file, not
                          // headers from the derived classes
using namespace std;

void main() {

  cout << "\n"
       << " testing protocol \n"
       << "==================\n";

  int i=0;

// note that clients can not make a protocol object, but fact-
// ory functions make anonymous derived class objects and give
// client access to them through a base class protocol pointer.


///////////////////////////////////////////////////////////////
// client uses the first derived class, knowing only the     //
// protocol language and how to call its object factory      //
///////////////////////////////////////////////////////////////

  protocol *bptr;
  bptr = protocol::make_d1();     // factory function creates
                                  // derived object

  bptr->putInt(2);                // use base class interface
  bptr->getInt();                 // derived class functions
  bptr->kill();

  cout << endl;

//

///////////////////////////////////////////////////////////////
// client uses the second derived class, knowing only the    //
// protocol language and how to call its object factory      //
///////////////////////////////////////////////////////////////

  bptr = protocol::make_d2();     // factory function creates
                                  // another derived object
  bptr->putInt(-3);
  bptr->getInt();
  bptr->kill();

  cout << '\n' << endl;

}
